unit Waveform; // Create 4th, after Debug, Comm, and Cal

// Modify waveform file to add test type
// 1/30/06 - Added MakeWaveFile, OpSpecArray
// 2/6/06 - Centered form on screen
// 2/3/07 - Alphabetized and arranged variables and procedures
// 3/30/09 - Various fixes for possible corrupt waveform file
// 1/13/10 - Code cleanup

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  xygraph, Debug,
  ExtCtrls, StdCtrls, Math, Menus, ComCtrls, Setup;

const
  MAXBUFF = 145000;  // For DataBuffer
  MAX_DATA = 60000; // 60000/2400 = 25 seconds

type
  IDType = array[0..7] of char;

  wave_hdr_type= packed record
    id: IDType;                         // "WAVEFORM"
    testtype: Char;
    ser_num: array [0..20] of char;     // Serial Number of Device
    units: array [0..12] of char;       // "Kiloamperes, etc"
    units_per_count: single;            // Units corresponding to each bit
    sec_per_sample: single;             // Sampling period
    bits: SmallInt;                     // 8, 10, 12, etc
    offset: SmallInt;                   // 0 = Unipolar, 2048 = 12 bit bipolar
    max_data: Integer;                 // Number of integers in buffer
    end;

  tOpSpec = (      // Operation specification type
    iAmplitude,    // Initial Peak Amplitude in Amperes
    iDuration,     // Duration of current pulse
    iStartPhase,   // Starting phase angle (degrees)
    iDecay,        // Percent drop off from start to end
    iOffDuration);  // Duration of off time after pulse = Reclose time

  tOpSpecArray = Array[1..5,tOpSpec] of Real;

type
  TfmWaveform = class(TForm)
    OpenWaveformDialog: TOpenDialog;
    SaveWaveformDialog: TSaveDialog;
    PaintBox1: TPaintBox;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ContractButton: TButton;
    ExpandButton: TButton;
    SaveButton: TButton;
    ExitButton: TButton;
    TrackBar1: TTrackBar;
    TStart: TEdit;
    TEnd: TEdit;
    CurStart: TEdit;
    CurSpan: TEdit;
    CurValue: TEdit;
    UpDown1: TUpDown;

    procedure Contract(Sender: TObject);
    procedure Expand(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormShow(Sender: TObject);
    procedure PaintBox1Click(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1Paint(Sender: TObject);
    procedure Panel1Click(Sender: TObject);
    procedure Panel1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Panel1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SaveButtonClick(Sender: TObject);
    procedure SetEnd(Sender: TObject);
    procedure SetCurStart(Sender: TObject);
    procedure SetCurSpan(Sender: TObject);
    procedure SetStart(Sender: TObject);
    procedure TrackBar1Change(Sender: TObject);
    procedure TrackBar1EndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure TrackBar1Exit(Sender: TObject);
    procedure TrackBar1KeyPress(Sender: TObject; var Key: Char);
    procedure TrackBar1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TrackBar1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TrackBar1OnMouseUp(Sender: TObject; var Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure ViewLeft(Sender: TObject);
    procedure ViewRight(Sender: TObject);

  private
    LastPosition: Integer;
    TimeStart: Single;            //Seconds
    TimeEnd: Single;              //Seconds
    MaxTime: Single;
    MinTime: Single;
    SampleStart: Integer;         //Ticks
    SampleEnd: Integer;           //Ticks
    CursorStart: Single;
    CursorEnd: Single;
    MaxY: Single;
    MinY: Single;
    Ydivisor: Single;
    procedure AdjustView;
    function  RMS( T1: single; T2: single ): single;
  public
    ADCrate: Real;
    SerNum: String;
    DataBuffer: array[0..MAXBUFF] of SmallInt; //60*2400=144000 for waveform
    DataBufferPtr: Integer;
    hdr: wave_hdr_type;
    MaxData : Integer;
    WaveFileName: String;
    MaxSample: Integer;
    OpSpecArray: tOpSpecArray;
    SecPerSample: Single;
    UnitsPerCount: Single;    //************ Added, was in Update
    WaveformSaved: Boolean;   // Set True when waveform is saved
    function  LoadFile(WaveFile: string): Boolean;
    procedure  MakeWaveFile(FileName: String; FileType: tTestMode; OpSpecArray: tOpSpecArray);
    function  ReadWaveform(WaveFile: string): Boolean;
    procedure  WriteWaveform(WaveFile: string);
  end;

var
  fmWaveform: TfmWaveform;

implementation

{$R *.DFM}

//************************* Adjust View *****************************
procedure TfmWaveform.AdjustView;
var       TimeSpan: Single;
begin
  TimeSpan := TimeEnd - TimeStart;
  TimeStart := MaxTime * (TrackBar1.Position/TrackBar1.Max);
  TimeEnd := TimeStart + TimeSpan;
  If TimeEnd > MaxTime then begin
    TimeEnd := MaxTime;
    TimeStart := MaxTime - TimeSpan; end;
  PaintBox1Paint(Self);
end;

//************************* Contract *****************************
procedure TfmWaveform.Contract(Sender: TObject);
var TimeSpan, TimeCenter: Single;
begin
  TimeSpan := (TimeEnd-TimeStart) * 2;
  If TimeSpan > MaxTime then TimeSpan := MaxTime;
  TimeCenter := (TimeEnd+TimeStart) / 2;
  TimeStart := TimeCenter - TimeSpan / 2;
  If TimeStart < 0 then TimeStart := 0;
  TimeEnd := TimeStart + TimeSpan;
  If TimeEnd > MaxTime then TimeEnd := MaxTime;
  PaintBox1Paint(Self);
end;

//************************* ExitButton *****************************
procedure TfmWaveform.ExitButtonClick(Sender: TObject);
begin
  fmWaveform.Close;
end;

//************************* Expand *****************************
procedure TfmWaveform.Expand(Sender: TObject);
//var TimeSpan, TimeCenter: Single;

begin
  TimeStart := CursorStart - 5*SecPerSample;
  If TimeStart <= 0.0 then
    TimeStart := 0.0;
  If CursorEnd > TimeStart+0.02 then
    TimeEnd := CursorEnd + 5*SecPerSample
  else
    TimeEnd := TimeStart + 0.01 + 5*SecPerSample;
  PaintBox1Paint(Self);
end;

//************************* Form *****************************
procedure TfmWaveform.FormActivate(Sender: TObject);
begin
  Ydivisor := Power(2,UpDown1.Position);
  MaxY :=  UnitsPerCount*2048/Ydivisor;  //******************
  MinY := -MaxY;
  PaintBox1Paint(Self);
end;

procedure TfmWaveform.FormCreate(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'Waveform Form Create' );
  ADCrate := 2400;
  SecPerSample := 1/ADCRate;
end;

procedure TfmWaveform.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//  xykeydown(key,shift);
end;

procedure TfmWaveform.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if TrackBar1.Position <> LastPosition then begin
    AdjustView;
    LastPosition := TrackBar1.Position;
  end;
end;

procedure TfmWaveform.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if TrackBar1.Position <> LastPosition then begin
    AdjustView;
    LastPosition := TrackBar1.Position;
  end;
end;

procedure TfmWaveform.FormShow(Sender: TObject);
begin
  fmDebug.AddDebugItem( 'Waveform FormShow' );
  PaintBox1Paint(Self);
  MaxTime := MAX_DATA/ADCRate;
  MinTime := 0;
  TimeStart := 0;            // Seconds
  TimeEnd := 1.0;            // Seconds
  SecPerSample := 1/ADCRate;
  CursorStart := TimeStart;  // Seconds
  CursorEnd := TimeEnd;      // Seconds
  MinY := -2048 * UnitsPerCount; 
  MaxY := 2048 * UnitsPerCount;
  WaveformSaved := False;
end;

//************************* LoadFile *****************************
function TfmWaveform.LoadFile(WaveFile: string): Boolean;
begin
  fmDebug.AddDebugItem( 'Waveform LoadFile' );
  if not ReadWaveform(WaveFile) then
    Result := False
  else begin
    Result := True;
    MaxTime := MaxData*SecPerSample;
    MinTime := 0;
    TimeStart := 0;
    TimeEnd := 1.00;                           //MaxTime;
    MinY := -2048;
    MaxY := 2048;
    CursorStart := TimeStart;
    CursorEnd := TimeEnd;
  end;
end;

//******************* Make waveform file from OpSpecArray *********************
procedure  TfmWaveform.MakeWaveFile(FileName: String; FileType: tTestMode; OpSpecArray: tOpSpecArray);
var
  hdr: wave_hdr_type;
  hWaveFile: Integer;
  i, j: Integer;
  Max_i: Real;
  msg: String;
  written: Integer;
  BuffPtr: Integer;
  SampleRate: Integer;
  RadiansPerSample: Real;
  StartRadians: Real;
  Amplitude: Real;
  Decay: Real;
  PulseWidth: Real;

begin
  fmDebug.AddDebugItem( 'MakeWaveFile' );
  SampleRate := 2400;
  hdr.id := 'WAVEFORM';
  if FileType = NORMAL then
    hdr.testtype := 'N'
  else
    hdr.testtype := 'M';
  hdr.ser_num := '***DEFAULT***';
  hdr.units := 'Amperes';
  hdr.units_per_count := 500/2048; // 500 A peak / 2048 counts
  hdr.sec_per_sample := 1/SampleRate;   //For 2400/sec ADCrate
  hdr.bits := 10;
  hdr.offset := 2048;

  RadiansPerSample := DegToRad(360 * 60 / SampleRate);
  hWaveFile := FileCreate(FileName);

  if( hWaveFile > 0 ) then begin
    hdr.max_data := MAX_DATA;  // Default size
    msg := 'Writing Waveform File: ' + FileName;
    written := FileWrite(hWaveFile, hdr, SizeOf(wave_hdr_type));
    // 0.1 seconds lead-in
    BuffPtr := 0;
    For i := 0 to Trunc(0.1*SampleRate) do begin
      DataBuffer[BuffPtr] := 0;
      inc(BuffPtr); end;
    For j := 1 to 5 do begin
      Amplitude := OpSpecArray[j,iAmplitude];
      PulseWidth := OpSpecArray[j,iDuration];
      Decay := OpSpecArray[j,iDecay];
      StartRadians := DegToRad(OpSpecArray[j,iStartPhase]);
      Max_i := PulseWidth*SampleRate;
      If Max_i >= 1 then begin
        For i := 0 to Trunc(Max_i) do begin
          DataBuffer[BuffPtr] :=
            Trunc( (1-(i/Max_i)*(1-Decay))*Amplitude*sin(StartRadians+i*RadiansPerSample));
          inc(BuffPtr); end;
        end;
      Max_i := OpSpecArray[j,iOffDuration]*SampleRate;
      If Max_i >= 1 then begin
        For i := 0 to Trunc(Max_i) do begin
          DataBuffer[BuffPtr] := 0;
          inc(BuffPtr); end;
        end;
      end;
    if( MessageDlg(msg, mtWarning,[mbOK,mbCancel], 0)) = mrOK then begin
      written := written + FileWrite( hWaveFile, DataBuffer, MAX_DATA);
      msg := Format( 'Bytes written %d', [written] );
      MessageDlg(msg, mtInformation,[mbOK], 0)
    end;
    FileClose( hWaveFile );
  end;
end;

//************************* PaintBox *****************************
procedure TfmWaveform.PaintBox1Click(Sender: TObject);
begin
  PaintBox1.Repaint;
end;

procedure TfmWaveform.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var yu: Single;
    StartTick, EndTick: Integer;
begin
  xymousedown(button,shift,x,y);
  If  ssLeft in shift then
    xyabstouser( x, y, CursorStart, yu )
  else if ssRight in shift then
    xyabstouser( x, y, CursorEnd, yu );
  StartTick := Round(CursorStart/SecPerSample);
  EndTick := Round(CursorEnd/SecPerSample);
  if (EndTick < StartTick) or ((EndTick-StartTick) < 1) then begin
    if ssRight in shift then
      StartTick := EndTick;
    EndTick := StartTick + 1;
    end;
  CursorStart := StartTick*SecPerSample;
  CursorEnd := EndTick*SecPerSample;
  PaintBox1.RePaint;
end;

procedure TfmWaveform.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var yu: Single;
begin
  xymousemove(shift,x,y);
  If  ssLeft in shift then
    xyabstouser( x, y, CursorStart, yu )
  else if ssRight in shift then
    xyabstouser( x, y, CursorEnd, yu );
  if (CursorEnd < CursorStart) or (Abs(CursorEnd-CursorStart)<SecPerSample) then
    CursorEnd := CursorStart + SecPerSample;
  PaintBox1.RePaint;
end;

procedure TfmWaveform.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  xymouseup(button,shift,x,y);
end;

procedure TfmWaveform.PaintBox1Paint(Sender: TObject);
var i : Integer;
    GridStart, GridEnd: Single;
begin
  try
  GridStart := TimeStart;           //Seconds
  if GridStart < 0 then
    GridStart := 0.0;
  GridEnd := TimeEnd;
  if (GridEnd - GridStart) < 0.02 then
    GridEnd := GridStart + 0.02;

  SampleStart := round(GridStart * ADCRate );   //Ticks
  SampleEnd := round(GridEnd * ADCRate );       //Ticks

  xycleargraph(paintbox1,clblack,clwhite,1.1);
  xystartgraph(0,100,0,100,50,10,10,50,clipon);

  xyxaxis(clwhite,GridStart,GridEnd,0,0,'',gridon,False,fixed);
  xyyaxis(clwhite,MinY,MaxY,0,0,'',1,gridon,False,fixed);
  paintbox1.canvas.pen.width := 2;
  paintbox1.canvas.pen.color := clYellow;
  xymove(SampleStart/ADCRate, UnitsPerCount*DataBuffer[SampleStart]);
  for i := SampleStart+1 to SampleEnd do
    begin xydraw(i/ADCRate, UnitsPerCount*DataBuffer[i]); xysymbol(1,0,1); end;

  paintbox1.canvas.pen.width := 2;                        //Draw Cursors
  xymove( CursorStart, MaxY );
  paintbox1.canvas.pen.color := clLime;
  xydraw( CursorStart,MinY );
  xymove( CursorEnd,MaxY );
  paintbox1.canvas.pen.color := clRed;
  xydraw( CursorEnd,MinY );

  TStart.Text := FormatFloat( '0.0000' , GridStart);
  TEnd.Text := FormatFloat( '0.0000' , GridEnd);
  If( MaxTime > MinTime ) then
  TrackBar1.Position := round(TrackBar1.Max*(GridStart/(MaxTime-MinTime)));
  LastPosition := TrackBar1.Position;
  If CursorStart > CursorEnd then
    CursorStart := CursorEnd-SecPerSample;
  If CursorStart < MinTime then
    CursorStart := MinTime;
  If CursorEnd > MaxTime then CursorEnd := MaxTime;
  CurStart.Text := FormatFloat ( '0.0000', CursorStart );
  CurSpan.Text := FormatFloat ( '0.0000', CursorEnd-CursorStart );
  CurValue.Text := FormatFloat ( '0.0000', RMS( CursorStart, CursorEnd ));
  except
  fmDebug.AddDebugItem( 'Waveform Paint Exception' );
  end;
end;

//************************* Panel *****************************
procedure TfmWaveform.Panel1Click(Sender: TObject);
begin
  PaintBox1.Repaint;
end;

procedure TfmWaveform.Panel1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if TrackBar1.Position <> LastPosition then begin
    AdjustView;
    LastPosition := TrackBar1.Position;
  end;

end;

procedure TfmWaveform.Panel1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if TrackBar1.Position <> LastPosition then begin
    AdjustView;
    LastPosition := TrackBar1.Position;
  end;

end;

//************************* Read Waveform *****************************
function TfmWaveform.ReadWaveform(WaveFile: string): Boolean;

type
  wave_char_file = file of char;
var
  id: IDType;
  hWaveFile: Integer;
  buff_ptr, ActualRead: Integer;

begin
  fmDebug.AddDebugItem( 'ReadWaveform' );
if FileExists( WaveFile )then begin //1
  For buff_ptr := 0 to MAX_DATA do
    DataBuffer[buff_ptr] := 0;
  id := 'WAVEFORM' ;
  hWaveFile := FileOpen(WaveFile, fmOpenRead);
  If hWaveFile > 0 then
    ActualRead := FileRead(hWaveFile, hdr, SizeOf(wave_hdr_type))
  else begin
    Result := False;
    FileClose( hWaveFile );
    Exit; end;
  fmDebug.AddDebugItem( 'Waveform Max Data = ' + IntToStr(hdr.max_data) );
  If (hdr.max_data > MAX_DATA) or (hdr.max_data < 100) then
    hdr.max_data := MAX_DATA;
  MaxData := hdr.max_data;

  if CompareStr(id, hdr.id)<>0 then begin
    MessageDlg( WaveFile + ' is Wrong File Type', mtError, [mbOK], 0);
    Result := False; end
  else begin
    ADCrate := 1/hdr.sec_per_sample;
    if (ADCrate < 500) or (ADCrate > 5000) then
      ADCrate := 2400;
    SecPerSample := 1/ADCrate;
    UnitsPerCount := hdr.units_per_count;
    buff_ptr := 0;
    Result := True;
    While ( buff_ptr < MaxData-1 ) do begin
      ActualRead := FileRead(hWaveFile, DataBuffer[buff_ptr], SizeOf(DataBuffer[buff_ptr]));
      inc(buff_ptr);
      if buff_ptr > MAXBUFF then break;
    end;
  end;
  FileClose( hWaveFile );
  end //1
else begin //1
  MessageDlg('File: ' + WaveFile + ' Not Found', mtError, [mbOK], 0);
  Result := False;
  end; //1
end;

//************************* RMS *****************************
function  TfmWaveform.RMS( T1: float; T2: float ): float;
var    rmsValue, rmsTime: single;
       sample: Integer;
       SampleCount: Integer;
begin
  rmsValue := 0;
  rmsTime := T1;
  SampleCount := 0;
  if SecPerSample <= 0 then
    SecPerSample := 1/ADCrate;
  sample := round( T1 / SecPerSample );
  if (T2-T1) < SecPerSample then
    RMS := Abs(UnitsPerCount*DataBuffer[sample])
  else begin
    while (rmsTime < T2 ) do begin
      rmsValue := rmsValue + Power( UnitsPerCount*DataBuffer[sample], 2 );
      sample := sample + 1;
      SampleCount := SampleCount + 1;
      rmsTime := rmsTime + SecPerSample; end;
    if SampleCount > 0 then
      RMS := sqrt( rmsValue / SampleCount )
    else
      RMS := 0;
  end;
end;

//************************* SaveButton *****************************
procedure TfmWaveform.SaveButtonClick(Sender: TObject);
var       CurrDir: String;
begin
  CurrDir := GetCurrentDir;
  SaveWaveformDialog.InitialDir := CurrDir;
  if( SaveWaveformDialog.Execute ) then begin
    WaveFileName := ExtractFileName(SaveWaveformDialog.FileName);
    WriteWaveform(WaveFileName); end;
  SetCurrentDir(CurrDir);  // added in case dir changed
end;

//************************* Set Procedures *****************************
procedure TfmWaveform.SetCurSpan(Sender: TObject);
begin
  CursorEnd := CursorStart + StrToFloat( CurSpan.Text );
  PaintBox1Paint(Self);
end;

procedure TfmWaveform.SetCurStart(Sender: TObject);
begin
  CursorStart := StrToFloat( CurStart.Text );
  PaintBox1Paint(Self);
end;

procedure TfmWaveform.SetEnd(Sender: TObject);
begin
  TimeEnd := StrToFloat(TEnd.Text);
  If TimeEnd > MaxTime then
    TimeEnd := MaxTime
  else if TimeEnd <= 0.05 then
    TimeEnd := 0.05;
  If TimeEnd < TimeStart then
    TimeEnd := TimeStart + 0.05;
  PaintBox1Paint(Self);
end;

procedure TfmWaveform.SetStart(Sender: TObject);
begin
  TimeStart := StrToFloat(TStart.Text);
  If TimeStart < 0.00 then
    TimeStart := 0.00
  else if TimeStart >= MaxTime then
    TimeStart := MaxTime;
  If TimeStart > TimeEnd then
    TimeEnd := TimeStart + 0.05;
  PaintBox1Paint(Self);
end;

//************************* TrackBar *****************************
procedure TfmWaveform.TrackBar1Change(Sender: TObject);
begin
  if TrackBar1.Position <> LastPosition then begin
    AdjustView;
    LastPosition := TrackBar1.Position;
  end;
end;

procedure TfmWaveform.TrackBar1EndDrag(Sender, Target: TObject; X,
  Y: Integer);
begin
   AdjustView;
end;

procedure TfmWaveform.TrackBar1Exit(Sender: TObject);
begin
  AdjustView;
end;

procedure TfmWaveform.TrackBar1KeyPress(Sender: TObject; var Key: Char);
begin
   AdjustView;
end;

procedure TfmWaveform.TrackBar1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//   AdjustView(Self);
end;

procedure TfmWaveform.TrackBar1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
//   AdjustView(Self);
end;

procedure TfmWaveform.TrackBar1OnMouseUp(Sender: TObject; var Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   AdjustView;
end;

//************************* UpDown *****************************
procedure TfmWaveform.UpDown1Click(Sender: TObject; Button: TUDBtnType);
begin
  Ydivisor := Power(2,UpDown1.Position);
  MaxY :=  UnitsPerCount*2048/Ydivisor;
  MinY := -MaxY;
  PaintBox1Paint(Self);
end;

//************************* Views *****************************
procedure TfmWaveform.ViewLeft(Sender: TObject);
var Shift: Single;
begin
  Shift := (TimeEnd-TimeStart) / 2;
  If TimeStart - Shift < 0 then begin
    TimeStart := 0;
    TimeEnd := Shift *2; end
  else begin
    TimeStart := TimeStart - Shift;
    TimeEnd := TimeEnd - Shift; end;
  PaintBox1Paint(Self);
end;

procedure TfmWaveform.ViewRight(Sender: TObject);
var Shift: Single;
begin
  Shift := (TimeEnd-TimeStart) / 2;
  if TimeEnd + Shift < MaxTime then begin
    TimeStart := TimeStart + Shift;
    TimeEnd := TimeEnd + Shift; end
  else begin
    TimeEnd := MaxTime;
    TimeStart := MaxTime - Shift * 2; end;
  PaintBox1Paint(Self);
end;

//************************* WriteWaveform *****************************
procedure TfmWaveform.WriteWaveform(WaveFile: string);

var
  hdr: wave_hdr_type;
  hWaveFile: Integer;
  i: Integer;
  msg: String;
  written: Integer;

begin
  fmDebug.AddDebugItem( 'WriteWaveform' );
  hdr.id := 'WAVEFORM';
  SerNum := InputBox( 'WaveFile Serial Number', 'Enter Ser Num or ID:', 'DEFAULT');
  SetLength(SerNum, strlen(hdr.ser_num) );
  for i := 1 to strlen(hdr.ser_num) do
    hdr.ser_num[i] := SerNum[i];
  hdr.units := 'Amperes';
  hdr.units_per_count := UnitsPerCount;
  hdr.sec_per_sample := 1/ADCrate;
  hdr.bits := 12;
  hdr.offset := 2048;

  if( not FileExists(WaveFile) ) then
    hWaveFile := FileCreate(WaveFile)
  else begin
    msg := 'Waveform File: ' + WaveFile + ' Exists. Overwrite?';
    if( MessageDlg(msg, mtWarning,[mbOK,mbCancel], 0) = mrOK ) then
      hWaveFile := FileOpen(WaveFile, fmOpenWrite)
    else
      hWaveFile := 0;
    end;
  if( hWaveFile > 0 ) then begin //1
    hdr.id := 'WAVEFORM' ;
    if fmSetup.TestMode = NORMAL then
      hdr.TestType := 'N'
    else if fmSetup.TestMode = MINPU then
      hdr.TestType := 'M'
    else
      hdr.TestType := '?';

    hdr.max_data := MAX_DATA;
    msg := 'Writing Waveform File: ' + WaveFile;
    if( MessageDlg(msg, mtWarning,[mbOK,mbCancel], 0) = mrOK ) then begin //2
      written := FileWrite(hWaveFile, hdr, SizeOf(wave_hdr_type));
      written := written + FileWrite( hWaveFile, DataBuffer, MAX_DATA);
      msg := Format( 'Bytes written %d', [written] );
      MessageDlg(msg, mtInformation,[mbOK], 0)
    end; //-2
    FileClose( hWaveFile );
  end; //-1
end;

end.
